include "pc82.inc"

dseg
SoundProc_: public SoundProc_
    defw 0

; void InitSound();
cseg
InitSound_: public InitSound_
    push h
        lxi h,0
        shld SoundProc_
    pop h
ret


; void CallSound();
CallSound:
pchl
CallSound_: public CallSound_
    push psw | push h
        lhld SoundProc_
        mov a,l
        ora h
        if nz
            call CallSound
            lxi h,0
            shld SoundProc_ 
        endif
    pop h | pop psw
ret


Pulse:
; in:
;   bc = cycle
    in PORTB
    ani not $20
    out PORTB
    push b
        do
            dcx b
            mov a,c
            ora b
        while nz | wend
    pop b
    in PORTB
    ori $20
    out PORTB
    push b
        do
            dcx b
            mov a,c
            ora b
        while nz | wend
    pop b
ret 

Tone:
; in:
;   bc = cycle
;   hl = length
    push d
        do
            call Pulse
            mov a,l | sub c | mov l,a
            mov a,h | sbb b | mov h,a
        while nc | wend
    pop d
ret 

Dummy:
; in:
;   bc = cycle
    push b
        do
            dcx b
            mov a,c
            ora b
        while nz | wend
    pop b
    push b
        do
            dcx b
            mov a,c
            ora b
        while nz | wend
    pop b
ret

Rest:
; in:
;   b = cycle
;   hl = length
    push d
        do
            call Dummy
            mov a,l | sub c | mov l,a
            mov a,h | sbb b | mov h,a
        while nc | wend
    pop d
ret 

MakeLangth:
    mov h,a
    mvi l,0
    mov d,h
    mov e,l
    dad h ;*2
    dad h ;*4
    dad d ;*5
ret

Melody:
; in
;   hl: address
    push psw | push d | push b
        do
            mov c,m | inx h
            mov b,m | inx h
            mov a,c
            ora a
        while nz
            if nz
                mov a,b
                ora a
                if nz
                    push h
                        mov a,c
                        dcr b
                        mov l,b
                        mvi h,0
                        dad h
                        lxi d,cycles
                        dad d
                        mov c,m
                        inx h
                        mov b,m
                    pop h
                    push h
                        call MakeLangth
                        call Tone
                    pop h
                else
                    push h
                        mov a,c
                        lxi b,100
                        call MakeLangth
                        call Rest
                    pop h
                endif
            endif
        wend
    pop b | pop d | pop psw
ret


; void Sound_Get();
Sound_Get:
    push psw | push h | push b
        lxi b,200
        lxi h,1000
        call Tone
    pop b | pop h | pop psw
ret 
Sound_Get_: public Sound_Get_
    push h
        lxi h,Sound_Get
        shld SoundProc_
    pop h
ret


; void Sound_Hit();
Sound_Hit:
    push psw | push b
        lxi b,300
        do
            Sound_Hit_.Loop:
            call Pulse
            dcx b
            mov a,b
            ora a
            jnz Sound_Hit_.Loop
            mov a,c
            cpi 150
        while nz | wend
    pop b | pop psw
ret 
Sound_Hit_: public Sound_Hit_
    push h
        lxi h,Sound_Hit
        shld SoundProc_
    pop h
ret


; ; void Sound_Miss();
; Sound_Miss:
;     push psw | push b
;         lxi b,150
;         do
;             Sound_Miss_.Loop:
;             call Pulse
;             inx b
;             mov a,b
;             ora a
;             jz Sound_Miss_.Loop
;             mov a,c
;             cpi low 300
;         while nz | wend
;     pop b | pop psw
; ret 
; Sound_Miss_:  public Sound_Miss_
;     push h
;         lxi h,Sound_Miss
;         shld SoundProc_
;     pop h
; ret


; void Sound_Loose();
Sound_Loose:
    push psw | push h | push b
        lxi b,600
        mvi l,10
        do
            call Pulse
            dcr l
        while nz| wend
    pop b | pop h | pop psw
ret 
Sound_Loose_: public Sound_Loose_
    push h
        lxi h,Sound_Loose
        shld SoundProc_
    pop h
ret


start_notes: defb 3,C4, 3,G4, 3,E4, 3,G4, 12,C5, 0
Sound_Start_: public Sound_Start_
    push h
        lxi h,start_notes
        call Melody
    pop h
ret


clear_notes: defb 3,C4, 3,E4, 3,G4, 3,D4, 3,F4, 3,A4, 3,E4, 3,G4, 3,B4, 9,C5,0
Sound_Clear_: public Sound_Clear_
    push h
        lxi h,clear_notes
        call Melody
    pop h
ret

over_notes: defb 3,C5, 3,G4, 3,E4, 3,C5,  3,B4, 3,G4, 3,E4, 3,B4,  6,A4, 6,B4,  12,C5, 0
Sound_GameOver_: public Sound_GameOver_
    push h
        lxi h,over_notes
        call Melody
    pop h
ret


C4 equ 1
C4S equ 2
D4 equ 3
D4S equ 4
E4 equ 5
F4 equ 6
F4S equ 7
G4 equ 8
G4S equ 9
A4 equ 10
A4S equ 11
B4 equ 12
C5 equ 13
C5S equ 14
D5 equ 15
D5S equ 16
E5 equ 17
F5 equ 18
F5S equ 19
G5 equ 20

cycles:
defw 176   ;C4
defw 166   ;C#4
defw 157   ;D4
defw 148   ;D#4
defw 140   ;E4
defw 132   ;F4
defw 124   ;F#4
defw 117   ;G4
defw 111   ;G#4
defw 104   ;A4
defw 99   ;A#4
defw 93   ;B4
defw 88   ;C5
defw 83   ;C#5
defw 78   ;D5
defw 74   ;D#5
defw 70   ;E5
defw 66   ;F5
defw 62   ;F#5
defw 58   ;G5
